from binarycpython.utils.dicts import AutoVivificationDict, inspect_dict, merge_dicts
from binarycpython.utils.ensemble import (
    binaryc_json_serializer,
    handle_ensemble_string_to_json,
)


# class test_return_binary_c_version_info(unittest.TestCase):
#     """
#     Unittests for return_binary_c_version_info
#     """

#     def test_not_parsed(self):
#         with Capturing() as output:
#             self._test_not_parsed()

#     def _test_not_parsed(self):
#         """
#         Test for the raw version_info output
#         """

#         version_info = return_binary_c_version_info(parsed=False)

#         self.assertTrue(isinstance(version_info, str))
#         self.assertIn("Build", version_info)
#         self.assertIn("REIMERS_ETA_DEFAULT", version_info)
#         self.assertIn("SIGMA_THOMPSON", version_info)

#     def test_parsed(self):
#         with Capturing() as output:
#             self._test_parsed()

#     def _test_parsed(self):
#         """
#         Test for the parssed version_info
#         """

#         # also tests the parse_version_info indirectly
#         version_info_parsed = return_binary_c_version_info(parsed=True)

#         self.assertTrue(isinstance(version_info_parsed, dict))
#         self.assertIn("isotopes", version_info_parsed.keys())
#         self.assertIn("argpairs", version_info_parsed.keys())
#         self.assertIn("ensembles", version_info_parsed.keys())
#         self.assertIn("macros", version_info_parsed.keys())
#         self.assertIn("elements", version_info_parsed.keys())
#         self.assertIn("dt_limits", version_info_parsed.keys())
#         self.assertIn("nucleosynthesis_sources", version_info_parsed.keys())
#         self.assertIn("miscellaneous", version_info_parsed.keys())



# class test_parse_binary_c_version_info(unittest.TestCase):
#     """
#     Unittests for function parse_binary_c_version_info
#     """

#     def test_1(self):
#         with Capturing() as output:
#             self._test_1()

#     def _test_1(self):
#         """
#         Test for the parsed versio info, more detailed
#         """

#         info = return_binary_c_version_info(parsed=False)
#         parsed_info = parse_binary_c_version_info(info)

#         self.assertIn("isotopes", parsed_info.keys())
#         self.assertIn("argpairs", parsed_info.keys())
#         self.assertIn("ensembles", parsed_info.keys())
#         self.assertIn("macros", parsed_info.keys())
#         self.assertIn("elements", parsed_info.keys())
#         self.assertIn("dt_limits", parsed_info.keys())
#         self.assertIn("nucleosynthesis_sources", parsed_info.keys())
#         self.assertIn("miscellaneous", parsed_info.keys())

#         self.assertIsNotNone(parsed_info["argpairs"])
#         self.assertIsNotNone(parsed_info["ensembles"])
#         self.assertIsNotNone(parsed_info["macros"])
#         self.assertIsNotNone(parsed_info["dt_limits"])
#         self.assertIsNotNone(parsed_info["miscellaneous"])

#         if parsed_info["macros"]["NUCSYN"] == "on":
#             self.assertIsNotNone(parsed_info["isotopes"])

#             if parsed_info["macros"]["NUCSYN_ID_SOURCES"] == "on":
#                 self.assertIsNotNone(parsed_info["nucleosynthesis_sources"])



class test_inspect_dict(unittest.TestCase):
    """
    Unittests for function inspect_dict
    """

    def test_compare_dict(self):
        with Capturing() as output:
            self._test_compare_dict()

    def _test_compare_dict(self):
        """
        Test checking if inspect_dict returns the correct structure by comparing it to known value
        """

        input_dict = {
            "int": 1,
            "float": 1.2,
            "list": [1, 2, 3],
            "function": os.path.isfile,
            "dict": {"int": 1, "float": 1.2},
        }
        output_dict = inspect_dict(input_dict)
        compare_dict = {
            "int": int,
            "float": float,
            "list": list,
            "function": os.path.isfile.__class__,
            "dict": {"int": int, "float": float},
        }
        self.assertTrue(compare_dict == output_dict)

    def test_compare_dict_with_print(self):
        with Capturing() as output:
            self._test_compare_dict_with_print()

    def _test_compare_dict_with_print(self):
        """
        Test checking output is printed
        """

        input_dict = {
            "int": 1,
            "float": 1.2,
            "list": [1, 2, 3],
            "function": os.path.isfile,
            "dict": {"int": 1, "float": 1.2},
        }
        output_dict = inspect_dict(input_dict, print_structure=True)


class test_merge_dicts(unittest.TestCase):
    """
    Unittests for function merge_dicts
    """

    def test_empty(self):
        with Capturing() as output:
            self._test_empty()

    def _test_empty(self):
        """
        Test merging an empty dict
        """

        input_dict = {
            "int": 1,
            "float": 1.2,
            "list": [1, 2, 3],
            "function": os.path.isfile,
            "dict": {"int": 1, "float": 1.2},
        }
        dict_2 = {}
        output_dict = merge_dicts(input_dict, dict_2)
        self.assertTrue(output_dict == input_dict)

    def test_unequal_types(self):
        with Capturing() as output:
            self._test_unequal_types()

    def _test_unequal_types(self):
        """
        Test merging unequal types: should raise valueError
        """

        dict_1 = {"input": 10}
        dict_2 = {"input": "hello"}

        self.assertRaises(ValueError, merge_dicts, dict_1, dict_2)

    def test_bools(self):
        with Capturing() as output:
            self._test_bools()

    def _test_bools(self):
        """
        Test merging dict with booleans
        """

        dict_1 = {"bool": True}
        dict_2 = {"bool": False}
        output_dict = merge_dicts(dict_1, dict_2)

        self.assertTrue(isinstance(output_dict["bool"], bool))
        self.assertTrue(output_dict["bool"])

    def test_ints(self):
        with Capturing() as _:
            self._test_ints()

    def _test_ints(self):
        """
        Test merging dict with ints
        """

        dict_1 = {"int": 2}
        dict_2 = {"int": 1}
        output_dict = merge_dicts(dict_1, dict_2)

        self.assertTrue(isinstance(output_dict["int"], int))
        self.assertEqual(output_dict["int"], 3)

    def test_floats(self):
        with Capturing() as output:
            self._test_floats()

    def _test_floats(self):
        """
        Test merging dict with floats
        """

        dict_1 = {"float": 4.5}
        dict_2 = {"float": 4.6}
        output_dict = merge_dicts(dict_1, dict_2)

        self.assertTrue(isinstance(output_dict["float"], float))
        self.assertEqual(output_dict["float"], 9.1)

    def test_lists(self):
        with Capturing() as output:
            self._test_lists()

    def _test_lists(self):
        """
        Test merging dict with lists
        """

        dict_1 = {"list": [1, 2]}
        dict_2 = {"list": [3, 4]}
        output_dict = merge_dicts(dict_1, dict_2)

        self.assertTrue(isinstance(output_dict["list"], list))
        self.assertEqual(output_dict["list"], [1, 2, 3, 4])

    def test_dicts(self):
        with Capturing() as _:
            self._test_dicts()

    def _test_dicts(self):
        """
        Test merging dict with dicts
        """

        dict_1 = {"dict": {"same": 1, "other_1": 2.0}}
        dict_2 = {"dict": {"same": 2, "other_2": [4.0]}}
        output_dict = merge_dicts(dict_1, dict_2)

        self.assertTrue(isinstance(output_dict["dict"], dict))
        self.assertEqual(
            output_dict["dict"], {"same": 3, "other_1": 2.0, "other_2": [4.0]}
        )

    def test_unsupported(self):
        with Capturing() as output:
            self._test_unsupported()

    def _test_unsupported(self):
        """
        Test merging dict with unsupported types. should raise ValueError
        """

        dict_1 = {"new": dummy("david")}
        dict_2 = {"new": dummy("gio")}

        # output_dict = merge_dicts(dict_1, dict_2)
        self.assertRaises(ValueError, merge_dicts, dict_1, dict_2)


class test_binaryc_json_serializer(unittest.TestCase):
    """
    Unittests for function binaryc_json_serializer
    """

    def test_not_function(self):
        with Capturing() as output:
            self._test_not_function()

    def _test_not_function(self):
        """
        Test passing an object that doesnt get turned in to a string
        """

        stringo = "hello"
        output = binaryc_json_serializer(stringo)
        self.assertTrue(stringo == output)

    def test_function(self):
        with Capturing() as output:
            self._test_function()

    def _test_function(self):
        """
        Test passing an object that gets turned in to a string: a function
        """

        string_of_function = str(os.path.isfile)
        output = binaryc_json_serializer(os.path.isfile)
        self.assertTrue(string_of_function == output)


class test_handle_ensemble_string_to_json(unittest.TestCase):
    """
    Unittests for function handle_ensemble_string_to_json
    """

    def test_1(self):
        with Capturing() as _:
            self._test_1()

    def _test_1(self):
        """
        Test passing string representation of a dictionary.
        """

        _ = str(os.path.isfile)
        input_string = '{"ding": 10, "list_example": [1,2,3]}'
        output_dict = handle_ensemble_string_to_json(input_string)

        self.assertTrue(isinstance(output_dict, dict))
        self.assertTrue(output_dict["ding"] == 10)
        self.assertTrue(output_dict["list_example"] == [1, 2, 3])


class test_AutoVivicationDict(unittest.TestCase):
    """
    Unittests for AutoVivicationDict
    """

    def test_add(self):
        """
        Tests to see if the adding is done correctly
        """

        result_dict = AutoVivificationDict()

        result_dict["a"]["b"]["c"] += 10

        self.assertEqual(result_dict["a"]["b"]["c"], 10)
        result_dict["a"]["b"]["c"] += 10
        self.assertEqual(result_dict["a"]["b"]["c"], 20)


